@@ -2,7 +2,7 @@ package Data::Processor;
 
 use strict;
 use 5.010_001;
-our $VERSION = '0.2.0';
+our $VERSION = '0.3.0';
 
 use Carp;
 use Data::Processor::Error::Collection;
@@ -14,14 +14,51 @@ use Data::Processor::ValidatorFactory;
 
 =head1 NAME
 
-THIS MODULE ONLY WORKS FOR A NARROW USE CASE RIGHT NOW. ALSO, INTERFACE CHANGES ARE LIKELY.
-
-Data::Processor - Transform Perl Data Structures, Validate Data against a Schema, Produce Data from a Schema, or produce documentation directly from information in the Data
+Data::Processor - Transform Perl Data Structures, Validate Data against a Schema, Produce Data from a Schema, or produce documentation directly from information in the Schema.
 
 =head1 SYNOPSIS
 
   use Data::Processor;
-  # XXX
+  my $schema = {
+    section => {
+        description => 'a section with a few members',
+        error_msg   => 'cannot find "section" in config',
+        members => {
+            foo => {
+                # value restriction either with a regex..
+                value => qr{f.*},
+                description => 'a string beginning with "f"'
+            },
+            bar => {
+                # ..or with a validator callback.
+                validator => sub {
+                    my $self   = shift;
+                    my $parent = shift;
+                    # undef is "no-error" -> success.
+                    no strict 'refs';
+                    return undef
+                        if $self->{value} == 42;
+                }
+            },
+            wuu => {
+                optional => 1
+            }
+        }
+    }
+  };
+
+  my $p = Data::Processor->new($schema);
+
+  my $data = {
+    section => {
+        foo => 'frobnicate',
+        bar => 42,
+        # "wuu" being optional, can be omitted..
+    }
+  };
+
+  my $error_collection = $p->validate($data, verbose=>0);
+  # no errors :-)
 
 =head1 DESCRIPTION
 
@@ -63,7 +100,7 @@ sub new{
 Validate the data against a schema. The schema either needs to be present
 already or be passed as an argument.
 
- my @errors = $processor->validate($data, verbose=>0);
+ my $error_collection = $processor->validate($data, verbose=>0);
 =cut
 sub validate{
     my $self = shift;
@@ -83,7 +120,10 @@ sub validate{
 
 =head2 validate_schema
 
-check that the schema is valid
+check that the schema is valid.
+This method gets called upon creation of a new Data::Processor object.
+
+ my $error_collection = $processor->validate_schema();
 
 =cut
 
@@ -191,6 +231,8 @@ transforms during validation.
 =cut
 # XXX make this traverse a data tree and transform everything
 # XXX across.
+# XXX Before hacking something here, think about factoring traversal out of
+# XXX D::P::Validator
 sub transform_data{
     my $self = shift;
     my $key  = shift;
@@ -201,14 +243,15 @@ sub transform_data{
 
 =head2 make_data
 
-UNIMPLEMENTED
+Writes a data template using the information found in the schema.
 
- my ($data, @errors) = $processor->make_data(data=>$data);
+ my $data = $processor->make_data(data=>$data);
 
 =cut
 sub make_data{
-    die 'unimplemented';
-    # XXX
+    my $self = shift;
+    my $entry_point = shift // $self->{schema};
+    return Data::Processor::Generator::make_data_template($entry_point);
 }
 
 =head2 make_pod
